import 'package:firebase_auth/firebase_auth.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Interface/iWatting.dart';
import 'package:wowit/firebaseService.dart';
import 'package:wowit/message.dart';
import 'package:wowit/sqlService.dart';
import 'package:wowit/methods.dart';
import 'package:flutter/material.dart';

class AuthController extends BaseController {
  final FirebaseAuth auth = FirebaseAuth.instance;

  Rx<User> _firebaseUser = Rx<User>();

  User get user => _firebaseUser.value;
  bool isInProcess = false;
  updateStatusButton() {
    isInProcess = true;
    update();
  }

  @override
  // ignore: must_call_super
  void onInit() {
    _firebaseUser.bindStream(auth.authStateChanges());
  }

  Future initAccountModel(bool triesLogin) async {
    String uidPhone = auth.currentUser.uid;
    String phoneNumber = auth.currentUser.phoneNumber;

    print('initAccountModel with uid:' + uidPhone);
    if (uidPhone == null || uidPhone == "") return;
    var rsData = await SQLService.execute("sp_getLogin",
        params: {"uidPhone": uidPhone, "phone": phoneNumber});

    var data = Methods.getList(rsData, 'info');

    print('data count :' + data.length.toString());

    if (data.length == 0) {
      print('Find not found this account with uidPhone :' +
          uidPhone +
          '==> update uid');
      var account = await SQLService.execute("sp_CheckExistsPhone",
          params: {"phone": phoneNumber});

//check exists this phone number in db
      if (Methods.getList(account, 'info').length > 0) {
        await SQLService.execute("sp_SyncAccount",
            params: {"uidPhone": uidPhone, "phone": phoneNumber});
      } else {
        //account not exists -> create new account
        await SQLService.execute("sp_CreateAccount",
            params: {"@uidPhone": uidPhone, 'phone': phoneNumber});
      }

      if (triesLogin) {
        await initAccountModel(false);
      }
    }
  }

  String strVerificationId = '';
  Future<void> signInWithPhone(
      final String phone, StatefulWidget screenOTP, Iwatting iwatting) async {
    this.updateStatusButton();

    iwatting.showWatting();

    final String phoneNumber = Methods.preparePhone(phone);
    auth.verifyPhoneNumber(
        phoneNumber: phoneNumber,
        timeout: Duration(seconds: 0),
        verificationCompleted: (AuthCredential authCredential) async {},
        verificationFailed: (FirebaseAuthException exception) {
          handleExceptionAuth(exception, iwatting);
          return '';
        },
        codeSent: (String verificationId, [int forceResendingToken]) async {
          iwatting.hideWatting();
          if (verificationId == '') {
            handleExceptionAuth(
                new Exception('VerificationId is empty'), iwatting);
            return;
          }
          iwatting.hideWatting();
          //show screen otp
          strVerificationId = verificationId;
          Get.to(() => screenOTP);
        },
        codeAutoRetrievalTimeout: (String verificationId) {
          iwatting.hideWatting();
          strVerificationId = verificationId;
        });
  }

  void verifyOTP(String otp, Widget homeScreen, Iwatting iwatting, HandleException handleException) {
    updateStatusButton();
    iwatting.showWatting();

    var _credential = PhoneAuthProvider.credential(
        verificationId: strVerificationId, smsCode: otp);
    auth.signInWithCredential(_credential).then((UserCredential result) async {
      await initAccountModel(true);
      iwatting.hideWatting();
      this.isInProcess = false;
      Get.offAll(() => homeScreen);
    }).catchError((e) {
      iwatting.hideWatting();

      if(handleException.handleExceptionAuth(e))
      {
        handleExceptionAuth(e, iwatting);
      }
      this.isInProcess = false;
      this.update();
      return "error";
    });
  }

  void handleExceptionAuth(dynamic e, Iwatting iwatting) {
    print('handleExceptionAuth :' + e.toString());
    if (e.toString().contains('The SMS code has expired')) {
      theSmsOTPTimeOut();
    } else if (e.toString().contains('Timed out waiting for SMS')) {
      theSMSCodeHasExpried();
    } else if (e.toString().contains(
        'We have blocked all requests from this device due to unusual activity')) {
      theBlockRequest();
    } else if (e.toString().contains(
        'The sms verification code used to create the phone auth credential is invalid')) {
      theSMSOTPInValid();
    } else {
      Get.snackbar(
        Message.getMessage('REGISTER_THONG_BAO'),
        e.toString(),
        snackPosition: SnackPosition.BOTTOM,
        duration: new Duration(milliseconds: 10000),
      );
    }

  }

  void theSMSOTPInValid() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_OPT_IN_VALID'),
      snackPosition: SnackPosition.BOTTOM,
      duration: new Duration(milliseconds: 10000),
    );
  }

  void theSmsOTPTimeOut() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_TIME_OUT'),
      snackPosition: SnackPosition.BOTTOM,
      duration: new Duration(milliseconds: 10000),
    );
  }

  void theBlockRequest() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_BLOCK_REQUEST'),
      snackPosition: SnackPosition.BOTTOM,
      duration: new Duration(milliseconds: 10000),
    );
  }

  void theSMSCodeHasExpried() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_EXPIRED'),
      snackPosition: SnackPosition.BOTTOM,
      duration: new Duration(milliseconds: 10000),
    );
  }

  Future<Map<String, List<Map<String, Object>>>> getLogin() async {
    var uidPhone = FirebaseAuth.instance.currentUser?.uid;
    var phone = FirebaseAuth.instance.currentUser?.phoneNumber;
    if (uidPhone == null || phone == null) return null;
    print("getLogin uidPhone: " + uidPhone + " , phone: " + phone);
    var rs = await SQLService.execute('sp_getLogin',
        params: {'phone': phone, 'uidPhone': uidPhone});
    var doc = Methods.getList(rs, 'info');
    if (doc.length == 0) return null;

    var data = doc[0];

    //REQUEST PERMISSION
    if (await FirebaseService.initializeFirebaseMessaging()) {
      String currentToken = Methods.getString(data, 'notifyTokenId');
      await FirebaseService.initNotifyToken(currentToken);
    }

    return rs;
  }

  @override
  Future<Map<String, List<Map<String, Object>>>> loadData() async{
    // TODO: implement loadData
    return  await this.getLogin();
  }
}
